##############################################################################
#
#    Recurrence Rules for Odoo
#    Copyright © 2015, 2018, 2022 XCG Consulting <https://www.xcg-consulting.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import dateutil.rrule
from odoo import fields, models


class RRuleSet(models.Model):
    """this model is used by all implementers who need to define a combination of other
    recurrence rules."""

    _name = "rrules.rruleset"
    _description = "Recurrence rule set"

    exclude_rule_ids = fields.Many2many(
        comodel_name="rrules.rrule",
        relation="rrules_rruleset_rrules_rrule_exclude_rel",
        column1="rruleset_id",
        column2="rrule_id",
        string="Exclude",
        help="Exclude the following rules",
    )

    include_rule_ids = fields.Many2many(
        comodel_name="rrules.rrule",
        relation="rrules_rruleset_rrules_rrule_include_rel",
        column1="rruleset_id",
        column2="rrule_id",
        string="Include",
        help="Include the following rules",
    )

    name = fields.Char(string="Name", translate=True)

    def get_rruleset(self, args=None):
        """
        :rtype: List of dateutil.rrule.rruleset.
        """

        self.ensure_one()

        rruleset = dateutil.rrule.rruleset()

        for rrule in self.include_rule_ids:
            rruleset.rrule(rrule.get_rrule(args=args))

        for rrule in self.exclude_rule_ids:
            rruleset.exrule(rrule.get_rrule(args=args))

        return rruleset
